@@ -1,4 +1,4 @@
-# Copyrights 2006-2008 by Mark Overmeer.
+# Copyrights 2006-2009 by Mark Overmeer.
 #  For other contributors see ChangeLog.
 # See the manual pages for details on the licensing terms.
 # Pod stripped from pm file by OODoc 1.05.
@@ -7,14 +7,14 @@ use strict;
 
 package XML::Compile::Util;
 use vars '$VERSION';
-$VERSION = '0.99';
+$VERSION = '1.00';
 
 use base 'Exporter';
 
 my @constants  = qw/XMLNS SCHEMA1999 SCHEMA2000 SCHEMA2001 SCHEMA2001i/;
 our @EXPORT    = qw/pack_type unpack_type/;
 our @EXPORT_OK =
-  ( qw/pack_id unpack_id odd_elements type_of_node/
+  ( qw/pack_id unpack_id odd_elements even_elements type_of_node escape/
   , @constants
   );
 our %EXPORT_TAGS = (constants => \@constants);
@@ -44,10 +44,8 @@ sub pack_id($$) { "$_[0]#$_[1]" }
 sub unpack_id($) { split /\#/, $_[0], 2 }
 
 
-sub odd_elements(@)
-{   my $i = 0;
-    map {$i++ % 2 ? $_ : ()} @_;
-}
+sub odd_elements(@)  { my $i = 0; map {$i++ % 2 ? $_ : ()} @_ }
+sub even_elements(@) { my $i = 0; map {$i++ % 2 ? () : $_} @_ }
 
 
 sub type_of_node($)
