@@ -1,4 +1,4 @@
-# Copyrights 2006-2008 by Mark Overmeer.
+# Copyrights 2006-2009 by Mark Overmeer.
 #  For other contributors see ChangeLog.
 # See the manual pages for details on the licensing terms.
 # Pod stripped from pm file by OODoc 1.05.
@@ -10,12 +10,12 @@ use lib '../XMLCompile/lib'  # test environment at home
 
 package TestTools;
 use vars '$VERSION';
-$VERSION = '0.99';
+$VERSION = '1.00';
 
 use base 'Exporter';
 
 use XML::LibXML;
-use XML::Compile::Util qw/SCHEMA2001/;
+use XML::Compile::Util ':constants';
 use XML::Compile::Tester;
 
 use Test::More;
@@ -26,15 +26,17 @@ use Data::Dumper qw/Dumper/;
 our @EXPORT = qw/
  $TestNS
  $SchemaNS
+ $SchemaNSi
  $dump_pkg
  test_rw
  /;
 
-our $TestNS   = 'http://test-types';
+our $TestNS    = 'http://test-types';
 set_default_namespace $TestNS;
 
-our $SchemaNS = SCHEMA2001;
-our $dump_pkg = 't::dump';
+our $SchemaNS  = SCHEMA2001;
+our $SchemaNSi = SCHEMA2001i;
+our $dump_pkg  = 't::dump';
 
 sub test_rw($$$$;$$)
 {   my ($schema, $test, $xml, $hash, $expect, $h2) = @_;
